/*
 * s_echelon.c - Echelon specific routines
 *
 *   Copyright (C) 2001 MyIRC Coding Team
 *
 *   This program is free softwmare; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: s_echelon.c,v 1.11 2001/09/28 04:30:10 psyex Exp $ */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include <sys/stat.h>
#include <utmp.h>
#include <fcntl.h>
#include "h.h"

static void millitime(long *sec, long *usec)
{
   struct timeval tv;
   gettimeofday(&tv, NULL);
   *sec = tv.tv_sec;
   *usec = tv.tv_usec;
}

/*
 * m_rping()
 * 
 * parv[0] = sender
 * parv[1] = pinged server
 * --[ Server <-> Server ]--
 * parv[2] = start time in seconds
 * parv[3] = start time in useconds
 * - Frosty
 */

int m_rping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
   long sec, usec;
   if (parc < 2) {
     sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "RPING");
     return 0;
   }
   if (MyClient(cptr)) {
     if (IsOper(cptr)) {
       aClient *acptr;
       if (!(acptr = find_server(parv[1], NULL))) { 
         sendto_one(sptr, err_str(ERR_NOSUCHSERVER), me.name, parv[0], parv[1]);
       } else if (IsMe(acptr)) {
         sendto_one(sptr, err_str(ERR_RPING), parv[1], parv[0]);
       } else {
         millitime(&sec, &usec);
         sendto_one(acptr, ":%s RPING %s %ld %ld", parv[0], parv[1], sec, usec);
       }
     } else {
       sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
     }
   }
   else if (hunt_server(cptr, sptr, ":%s RPING %s %s %s", 1, parc, parv) == HUNTED_ISME) {
     sendto_one(sptr, ":%s RPONG %s %s %s", me.name, parv[0], parv[2], parv[3]);
   }
   return 0;
}

/*
 * m_rpong()
 * 
 * parv[0] = sender (pinged server)
 * parv[1] = client (rping sender)
 * --[ Server <-> Server ]--
 * parv[2] = start time in seconds
 * parv[3] = start time in useconds
 * - Frosty
 */

int m_rpong(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
   aClient *acptr;
   long sec, usec;
   if (!IsServer(sptr)) return 0;
   // Only server can RPONG, so if user left, drop silently...
   if(!(acptr = find_person(parv[1], NULL))) return 0;
   if(!MyClient(acptr)) sendto_one(cptr, ":%s RPONG %s %s %s", parv[0], parv[1], parv[2], parv[3]);
   else {
     millitime(&sec, &usec);
     sec -= atol(parv[2]);
     usec -= atol(parv[3]) / 1000;
     sendto_one(acptr, rpl_str(RPL_RPING), parv[0], parv[1], parv[0], sec, usec);
   }
   return 0;
}

/*
 * findConfMatchHost()
 * Match some lines out of ircd.conf with host
 * - PsyeX
 */

int findConfMatchHost(char *host, aConfList *List1, aConfList *List2, aConfList *List3)
{
   aConfItem *tmp;
   aConfList *list;
   char rev[HOSTLEN + 1];
   char name[2];   

   sprintf(name, "*");
   if (strlen(host) > (size_t) HOSTLEN) return (0);
   reverse(rev, host);
   list = List2;
   while ((tmp = find_matching_conf(list, rev)) != NULL) {
      if (tmp->name && (!name || !match(tmp->name, name))) return (tmp ? -1 : 0);
      list = NULL;
   }
   list = List1;
   while ((tmp = find_matching_conf(list, host)) != NULL) {
      if (tmp->name && (!name || !match(tmp->name, name))) return (tmp ? -1 : 0);
      list = NULL;
   }
   list = List3;
   while ((tmp = l_find_matching_conf(list, host)) != NULL) {
      if (tmp->host && tmp->name && (!name || !match(tmp->name, name))) return (tmp ? -1 : 0);
      list = NULL;
   }
   return (tmp ? -1 : 0);
}

int findFline(char *host)
{
   return findConfMatchHost(host, &FList1, &FList2, &FList3);
}

